@@ -4,8 +4,8 @@
  * Copyright (c) 2016      Mellanox Technologies, Inc.
  *                         All rights reserved.
  * Copyright (c) 2016      IBM Corporation.  All rights reserved.
- * Copyright (c) 2021-2022 Nanook Consulting  All rights reserved.
- * Copyright (c) 2022      Triad National Security, LLC. All rights reserved.
+ * Copyright (c) 2021-2025 Nanook Consulting  All rights reserved.
+ * Copyright (c) 2022-2024 Triad National Security, LLC. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -56,7 +56,7 @@ static void atrkdes(pmix_attribute_trk_t *p)
     if (NULL != p->function) {
         free(p->function);
     }
-    pmix_argv_free(p->attrs);
+    PMIx_Argv_free(p->attrs);
 }
 static PMIX_CLASS_INSTANCE(pmix_attribute_trk_t, pmix_list_item_t, atrkcon, atrkdes);
 
@@ -73,15 +73,17 @@ PMIX_EXPORT void pmix_init_registered_attrs(void)
 
         /* cycle across the dictionary and load a hash
          * table with translations of key -> index */
-        for (n=0; UINT32_MAX != pmix_dictionary[n].index; n++) {
+        pmix_pointer_array_set_size(pmix_globals.keyindex.table, PMIX_INDEX_BOUNDARY);  // minimize realloc's
+        for (n=0; n < PMIX_INDEX_BOUNDARY; n++) {
             p = (pmix_regattr_input_t*)pmix_malloc(sizeof(pmix_regattr_input_t));
             p->index = pmix_dictionary[n].index;
             p->name = strdup(pmix_dictionary[n].name);
             p->string = strdup(pmix_dictionary[n].string);
             p->type = pmix_dictionary[n].type;
-            p->description = pmix_argv_copy(pmix_dictionary[n].description);
-            pmix_hash_register_key(p->index, p);
+            p->description = PMIx_Argv_copy(pmix_dictionary[n].description);
+            pmix_hash_register_key(p->index, p, &pmix_globals.keyindex);
         }
+        pmix_globals.keyindex.next_id = PMIX_INDEX_BOUNDARY;
         initialized = true;
     }
 }
@@ -117,7 +119,7 @@ static pmix_status_t process_reg(char *level, char *function, char **attrs)
     pmix_list_append(lst, &fnptr->super);
     fnptr->function = strdup(function);
     if (NULL != attrs) {
-        fnptr->attrs = pmix_argv_copy(attrs);
+        fnptr->attrs = PMIx_Argv_copy(attrs);
     }
     return PMIX_SUCCESS;
 }
@@ -537,6 +539,7 @@ PMIX_EXPORT pmix_status_t pmix_register_tool_attrs(void)
 }
 
 /*****   PROCESS QUERY ATTRS    *****/
+
 static void _get_attrs(pmix_list_t *lst, pmix_info_t *info, pmix_list_t *attrs)
 {
     pmix_attribute_trk_t *trk, *tptr;
@@ -544,53 +547,73 @@ static void _get_attrs(pmix_list_t *lst, pmix_info_t *info, pmix_list_t *attrs)
     pmix_data_array_t *darray;
     pmix_regattr_t *regarray;
     size_t m, nattr;
-    char **fns;
+    char **fns = NULL;
     const pmix_regattr_input_t *dptr;
 
     /* the value in the info is a comma-delimited list of
      * functions whose attributes are being requested */
-    fns = pmix_argv_split(info->value.data.string, ',');
+    if (NULL != info) {
+        fns = PMIx_Argv_split(info->key, ',');
+    }
 
     /* search the list for these functions */
     PMIX_LIST_FOREACH (tptr, attrs, pmix_attribute_trk_t) {
-        trk = NULL;
-        for (m = 0; NULL != fns[m] && NULL == trk; m++) {
-            if (0 == strcmp(fns[m], tptr->function) || 0 == strcmp(fns[m], "all")) {
-                trk = tptr;
-                break;
+        if (NULL == fns) {
+            // want them all - ignore if no attrs
+            if (NULL == tptr->attrs) {
+                continue;
+            }
+        } else {
+            trk = NULL;
+            for (m = 0; NULL != fns[m]; m++) {
+                if (0 == strcmp(fns[m], tptr->function) || 0 == strcmp(fns[m], "all")) {
+                    trk = tptr;
+                    break;
+                }
+            }
+            if (NULL == trk || NULL == trk->attrs) {
+                /* function wasn't found - no attrs
+                 * registered for it */
+                continue;
             }
-        }
-        if (NULL == trk || NULL == trk->attrs) {
-            /* function wasn't found - no attrs
-             * registered for it */
-            continue;
         }
         /* add the found attrs to the results */
         ip = PMIX_NEW(pmix_infolist_t);
         PMIX_LOAD_KEY(ip->info.key, tptr->function);
         /* create the data array to hold the results */
-        nattr = pmix_argv_count(tptr->attrs);
-        PMIX_DATA_ARRAY_CREATE(darray, nattr, PMIX_REGATTR);
-        ip->info.value.type = PMIX_DATA_ARRAY;
-        ip->info.value.data.darray = darray;
-        regarray = (pmix_regattr_t *) darray->array;
-        for (m = 0; m < nattr; m++) {
-            regarray[m].name = strdup(tptr->attrs[m]);
-            PMIX_LOAD_KEY(regarray[m].string, pmix_attributes_lookup(tptr->attrs[m]));
-            dptr = pmix_attributes_lookup_term(tptr->attrs[m]);
-            if (NULL == dptr) {
-                PMIX_RELEASE(ip);
-                return;
+        nattr = PMIx_Argv_count(tptr->attrs);
+        if (0 == nattr || (1 == nattr &&
+                           (0 == strcmp("N/A", tptr->attrs[0]) ||
+                            0 == strcmp("NONE", tptr->attrs[0])))) {
+            nattr = 1;
+            PMIX_DATA_ARRAY_CREATE(darray, nattr, PMIX_REGATTR);
+            ip->info.value.type = PMIX_DATA_ARRAY;
+            ip->info.value.data.darray = darray;
+            regarray = (pmix_regattr_t *) darray->array;
+            regarray[0].name = strdup("NONE");
+        } else {
+            PMIX_DATA_ARRAY_CREATE(darray, nattr, PMIX_REGATTR);
+            ip->info.value.type = PMIX_DATA_ARRAY;
+            ip->info.value.data.darray = darray;
+            regarray = (pmix_regattr_t *) darray->array;
+            for (m = 0; m < nattr; m++) {
+                regarray[m].name = strdup(tptr->attrs[m]);
+                PMIX_LOAD_KEY(regarray[m].string, pmix_attributes_lookup(tptr->attrs[m]));
+                dptr = pmix_attributes_lookup_term(tptr->attrs[m]);
+                if (NULL == dptr) {
+                    PMIX_RELEASE(ip);
+                    return;
+                }
+                regarray[m].type = dptr->type;
+                regarray[m].description = PMIx_Argv_copy(dptr->description);
             }
-            regarray[m].type = dptr->type;
-            regarray[m].description = pmix_argv_copy(dptr->description);
         }
         pmix_list_append(lst, &ip->super);
     }
-    pmix_argv_free(fns);
+    PMIx_Argv_free(fns);
 }
 
-static void _get_fns(pmix_list_t *lst, pmix_info_t *info, pmix_list_t *attrs)
+static void _get_fns(pmix_list_t *lst, char *key, pmix_list_t *attrs)
 {
     pmix_attribute_trk_t *tptr;
     pmix_infolist_t *ip;
@@ -598,262 +621,190 @@ static void _get_fns(pmix_list_t *lst, pmix_info_t *info, pmix_list_t *attrs)
 
     /* search the list for these functions */
     PMIX_LIST_FOREACH (tptr, attrs, pmix_attribute_trk_t) {
-        pmix_argv_append_nosize(&fns, tptr->function);
+        PMIx_Argv_append_nosize(&fns, tptr->function);
     }
-    if (0 < pmix_argv_count(fns)) {
+    if (0 < PMIx_Argv_count(fns)) {
         ip = PMIX_NEW(pmix_infolist_t);
-        tmp = pmix_argv_join(fns, ',');
-        PMIX_INFO_LOAD(&ip->info, info->key, tmp, PMIX_STRING);
+        tmp = PMIx_Argv_join(fns, ',');
+        PMIX_INFO_LOAD(&ip->info, key, tmp, PMIX_STRING);
         pmix_list_append(lst, &ip->super);
-        pmix_argv_free(fns);
+        PMIx_Argv_free(fns);
     }
 }
 
-static void _local_relcb(void *cbdata)
+PMIX_EXPORT void pmix_attrs_query_support(pmix_query_caddy_t *cd,
+                                          pmix_query_t *query,
+                                          pmix_list_t *unresolved)
 {
-    pmix_query_caddy_t *cd = (pmix_query_caddy_t *) cbdata;
-    PMIX_RELEASE(cd);
-}
-
-static void relcbfunc(void *cbdata)
-{
-    pmix_shift_caddy_t *cd = (pmix_shift_caddy_t *) cbdata;
-
-    pmix_output_verbose(2, pmix_globals.debug_output, "pmix:query release callback");
-
-    if (NULL != cd->info) {
-        PMIX_INFO_FREE(cd->info, cd->ninfo);
-    }
-    PMIX_RELEASE(cd);
-}
-static void query_cbfunc(struct pmix_peer_t *peer, pmix_ptl_hdr_t *hdr,
-                         pmix_buffer_t *buf, void *cbdata)
-{
-    pmix_query_caddy_t *cd = (pmix_query_caddy_t *) cbdata;
-    pmix_status_t rc;
-    pmix_shift_caddy_t *results;
-    int cnt;
-    PMIX_HIDE_UNUSED_PARAMS(hdr);
-
-    pmix_output_verbose(2, pmix_globals.debug_output, "pmix:attrs:query cback from server");
-
-    results = PMIX_NEW(pmix_shift_caddy_t);
-
-    /* unpack the status */
-    cnt = 1;
-    PMIX_BFROPS_UNPACK(rc, peer, buf, &results->status, &cnt, PMIX_STATUS);
-    if (PMIX_SUCCESS != rc) {
-        results->status = rc;
-        goto complete;
-    }
-    if (PMIX_SUCCESS != results->status) {
-        goto complete;
-    }
-
-    /* unpack any returned data */
-    cnt = 1;
-    PMIX_BFROPS_UNPACK(rc, peer, buf, &results->ninfo, &cnt, PMIX_SIZE);
-    if (PMIX_SUCCESS != rc) {
-        results->status = rc;
-        goto complete;
-    }
-    if (0 < results->ninfo) {
-        PMIX_INFO_CREATE(results->info, results->ninfo);
-        cnt = results->ninfo;
-        PMIX_BFROPS_UNPACK(rc, peer, buf, results->info, &cnt, PMIX_INFO);
-        if (PMIX_SUCCESS != rc) {
-            results->status = rc;
-            goto complete;
-        }
-    }
-
-complete:
-    pmix_output_verbose(2, pmix_globals.debug_output, "pmix:query cback from server releasing");
-    /* release the caller */
-    if (NULL != cd->cbfunc) {
-        cd->cbfunc(results->status, results->info, results->ninfo, cd->cbdata, relcbfunc, results);
-    }
-    PMIX_RELEASE(cd);
-}
-
-PMIX_EXPORT void pmix_attrs_query_support(int sd, short args, void *cbdata)
-{
-    pmix_query_caddy_t *cd = (pmix_query_caddy_t *) cbdata;
-    pmix_infolist_t *info, *head;
+    pmix_querylist_t *qry;
     pmix_list_t kyresults;
+    pmix_infolist_t *info;
     size_t n, m, p;
     pmix_info_t *iptr;
     pmix_data_array_t *darray;
-    pmix_buffer_t *msg;
-    pmix_cmd_t cmd = PMIX_QUERY_CMD;
     pmix_status_t rc;
-    PMIX_HIDE_UNUSED_PARAMS(sd, args);
+    pmix_kval_t *kv;
+    char **cache = NULL;
 
-    PMIX_ACQUIRE_THREAD(&pmix_global_lock);
+    // this is called from within an event, so no need to lock
 
-    for (n = 0; n < cd->nqueries; n++) {
-        if (0 != strcmp(cd->queries[n].keys[0], PMIX_QUERY_ATTRIBUTE_SUPPORT)) {
-            /* skip this one */
-            continue;
+    // if the qualifiers are NULL, then they want it all
+    if (NULL == query->qualifiers) {
+        PMIX_CONSTRUCT(&kyresults, pmix_list_t);
+        /* everyone has access to the client attrs */
+        _get_attrs(&kyresults, NULL, &client_attrs);
+        /* everyone has access to the client functions */
+        _get_fns(&kyresults, PMIX_CLIENT_FUNCTIONS, &client_attrs);
+       /* if I am a server... */
+        if (PMIX_PEER_IS_SERVER(pmix_globals.mypeer)) {
+            // add in my attrs and fns
+            _get_attrs(&kyresults, NULL, &server_attrs);
+            _get_fns(&kyresults, PMIX_SERVER_FUNCTIONS, &server_attrs);
+            // and add in my hosts
+            _get_attrs(&kyresults, NULL, &host_attrs);
+            _get_fns(&kyresults, PMIX_HOST_FUNCTIONS, &host_attrs);
+        } else {
+            PMIx_Argv_append_nosize(&cache, PMIX_SERVER_ATTRIBUTES);
+            PMIx_Argv_append_nosize(&cache, PMIX_SERVER_FUNCTIONS);
+            PMIx_Argv_append_nosize(&cache, PMIX_HOST_ATTRIBUTES);
+            PMIx_Argv_append_nosize(&cache, PMIX_HOST_FUNCTIONS);
         }
-        head = NULL;
-        for (m = 0; m < cd->queries[n].nqual; m++) {
-            PMIX_CONSTRUCT(&kyresults, pmix_list_t);
-            if (NULL == cd->queries[n].qualifiers
-                || PMIX_CHECK_KEY(&cd->queries[n].qualifiers[m], PMIX_CLIENT_ATTRIBUTES)) {
-                /* everyone has access to the client attrs */
-                _get_attrs(&kyresults, &cd->queries[n].qualifiers[m], &client_attrs);
-            }
-            if (NULL == cd->queries[n].qualifiers
-                || PMIX_CHECK_KEY(&cd->queries[n].qualifiers[m], PMIX_CLIENT_FUNCTIONS)) {
-                /* everyone has access to the client functions */
-                _get_fns(&kyresults, &cd->queries[n].qualifiers[m], &client_attrs);
-            }
-            if (NULL == cd->queries[n].qualifiers
-                || PMIX_CHECK_KEY(&cd->queries[n].qualifiers[m], PMIX_SERVER_ATTRIBUTES)) {
-                /* if I am a server, add in my attrs */
-                if (PMIX_PEER_IS_SERVER(pmix_globals.mypeer)) {
-                    _get_attrs(&kyresults, &cd->queries[n].qualifiers[m], &server_attrs);
-                } else {
-                    /* we need to ask our server for them */
-                    PMIX_LIST_DESTRUCT(&kyresults);
-                    goto query;
-                }
-            }
-            if (NULL == cd->queries[n].qualifiers
-                || PMIX_CHECK_KEY(&cd->queries[n].qualifiers[m], PMIX_SERVER_FUNCTIONS)) {
-                /* if I am a server, add in my fns */
-                if (PMIX_PEER_IS_SERVER(pmix_globals.mypeer)) {
-                    _get_fns(&kyresults, &cd->queries[n].qualifiers[m], &server_attrs);
-                } else {
-                    /* we need to ask our server for them */
-                    PMIX_LIST_DESTRUCT(&kyresults);
-                    goto query;
-                }
-            }
-            if (NULL == cd->queries[n].qualifiers
-                || PMIX_CHECK_KEY(&cd->queries[n].qualifiers[m], PMIX_TOOL_ATTRIBUTES)) {
-                if (PMIX_PEER_IS_TOOL(pmix_globals.mypeer)) {
-                    _get_attrs(&kyresults, &cd->queries[n].qualifiers[m], &tool_attrs);
-                }
-            }
-            if (NULL == cd->queries[n].qualifiers
-                || PMIX_CHECK_KEY(&cd->queries[n].qualifiers[m], PMIX_TOOL_FUNCTIONS)) {
-                if (PMIX_PEER_IS_TOOL(pmix_globals.mypeer)) {
-                    _get_fns(&kyresults, &cd->queries[n].qualifiers[m], &tool_attrs);
-                }
-            }
-            if (NULL == cd->queries[n].qualifiers
-                || PMIX_CHECK_KEY(&cd->queries[n].qualifiers[m], PMIX_HOST_ATTRIBUTES)) {
-                /* if I am a server, add in the host's */
-                if (PMIX_PEER_IS_SERVER(pmix_globals.mypeer)) {
-                    _get_attrs(&kyresults, &cd->queries[n].qualifiers[m], &host_attrs);
-                } else {
-                    /* we need to ask our server for them */
-                    PMIX_LIST_DESTRUCT(&kyresults);
-                    goto query;
-                }
-            }
-            if (NULL == cd->queries[n].qualifiers
-                || PMIX_CHECK_KEY(&cd->queries[n].qualifiers[m], PMIX_HOST_FUNCTIONS)) {
-                /* if I am a server, add in the host's */
-                if (PMIX_PEER_IS_SERVER(pmix_globals.mypeer)) {
-                    _get_fns(&kyresults, &cd->queries[n].qualifiers[m], &host_attrs);
-                } else {
-                    /* we need to ask our server for them */
-                    PMIX_LIST_DESTRUCT(&kyresults);
-                    goto query;
-                }
-            }
-            if (0 < (p = pmix_list_get_size(&kyresults))) {
-                head = PMIX_NEW(pmix_infolist_t);
-                PMIX_LOAD_KEY(head->info.key, cd->queries[n].keys[m]);
-                head->info.value.type = PMIX_DATA_ARRAY;
-                /* create the data array to hold the results */
-                PMIX_DATA_ARRAY_CREATE(darray, p, PMIX_INFO);
-                head->info.value.data.darray = darray;
-                iptr = (pmix_info_t *) darray->array;
-                p = 0;
-                PMIX_LIST_FOREACH (info, &kyresults, pmix_infolist_t) {
-                    PMIX_INFO_XFER(&iptr[p], &info->info);
-                    ++p;
-                }
-                pmix_list_append(&cd->results, &head->super);
-            }
-            PMIX_LIST_DESTRUCT(&kyresults);
+        // if I am a tool...
+        if (PMIX_PEER_IS_TOOL(pmix_globals.mypeer)) {
+            // add in my attrs and fns
+            _get_attrs(&kyresults, NULL, &tool_attrs);
+            _get_fns(&kyresults, PMIX_TOOL_FUNCTIONS, &tool_attrs);
         }
-    }
-    /* prep the response by converting the list
-     * of results into an array */
-    if (0 < (cd->ninfo = pmix_list_get_size(&cd->results))) {
-        PMIX_INFO_CREATE(cd->info, cd->ninfo);
-        n = 0;
-        PMIX_LIST_FOREACH (info, &cd->results, pmix_infolist_t) {
-            PMIX_INFO_XFER(&cd->info[n], &info->info);
-            ++n;
+        // collect the results
+        if (0 < (p = pmix_list_get_size(&kyresults))) {
+            PMIX_KVAL_NEW(kv, PMIX_QUERY_ATTRIBUTE_SUPPORT);
+            kv->value->type = PMIX_DATA_ARRAY;
+            /* create the data array to hold the results */
+            PMIX_DATA_ARRAY_CREATE(darray, p, PMIX_INFO);
+            kv->value->data.darray = darray;
+            iptr = (pmix_info_t *) darray->array;
+            p = 0;
+            PMIX_LIST_FOREACH (info, &kyresults, pmix_infolist_t) {
+                PMIX_INFO_XFER(&iptr[p], &info->info);
+                ++p;
+            }
+            pmix_list_append(&cd->results, &kv->super);
+        }
+        PMIX_LIST_DESTRUCT(&kyresults);
+        if (NULL != cache) {
+            goto request;
         }
-        cd->status = PMIX_SUCCESS;
-    } else {
-        cd->status = PMIX_ERR_NOT_FOUND;
-    }
-    PMIX_RELEASE_THREAD(&pmix_global_lock);
-    goto release;
-
-query:
-    /* if we aren't connected, don't attempt to send */
-    if (!pmix_globals.connected) {
-        PMIX_RELEASE_THREAD(&pmix_global_lock);
-        cd->status = PMIX_ERR_NOT_FOUND;
-        goto release;
-    }
-    PMIX_RELEASE_THREAD(&pmix_global_lock);
-
-    /* relay this request to the server */
-    msg = PMIX_NEW(pmix_buffer_t);
-    PMIX_BFROPS_PACK(rc, pmix_client_globals.myserver, msg, &cmd, 1, PMIX_COMMAND);
-    if (PMIX_SUCCESS != rc) {
-        PMIX_RELEASE(msg);
-        cd->status = rc;
-        goto release;
-    }
-    PMIX_BFROPS_PACK(rc, pmix_client_globals.myserver, msg, &cd->nqueries, 1, PMIX_SIZE);
-    if (PMIX_SUCCESS != rc) {
-        PMIX_RELEASE(msg);
-        cd->status = rc;
-        goto release;
-    }
-    PMIX_BFROPS_PACK(rc, pmix_client_globals.myserver, msg, cd->queries, cd->nqueries, PMIX_QUERY);
-    if (PMIX_SUCCESS != rc) {
-        PMIX_RELEASE(msg);
-        cd->status = rc;
-        goto release;
-    }
-
-    pmix_output_verbose(2, pmix_globals.debug_output, "pmix:query sending to server");
-    PMIX_PTL_SEND_RECV(rc, pmix_client_globals.myserver, msg, query_cbfunc, (void *) cd);
-    if (PMIX_SUCCESS != rc) {
-        cd->status = rc;
-        goto release;
-    }
-    return;
-
-release:
-    if (NULL != cd->cbfunc) {
-        cd->cbfunc(cd->status, cd->info, cd->ninfo, cd, _local_relcb, cd);
         return;
     }
 
-    PMIX_RELEASE(cd);
+    // otherwise, search for the ones they want
+    for (m = 0; m < query->nqual; m++) {
+        PMIX_CONSTRUCT(&kyresults, pmix_list_t);
+        if (PMIX_CHECK_KEY(&query->qualifiers[m], PMIX_CLIENT_ATTRIBUTES)) {
+            /* everyone has access to the client attrs */
+            _get_attrs(&kyresults, &query->qualifiers[m], &client_attrs);
+
+        } else if (PMIX_CHECK_KEY(&query->qualifiers[m], PMIX_CLIENT_FUNCTIONS)) {
+            /* everyone has access to the client functions */
+            _get_fns(&kyresults, PMIX_CLIENT_FUNCTIONS, &client_attrs);
+
+        } else if (PMIX_CHECK_KEY(&query->qualifiers[m], PMIX_SERVER_ATTRIBUTES)) {
+            /* if I am a server, add in my attrs */
+            if (PMIX_PEER_IS_SERVER(pmix_globals.mypeer)) {
+                _get_attrs(&kyresults, &query->qualifiers[m], &server_attrs);
+            } else {
+                PMIx_Argv_append_nosize(&cache, PMIX_SERVER_ATTRIBUTES);
+            }
+
+        } else if (PMIX_CHECK_KEY(&query->qualifiers[m], PMIX_SERVER_FUNCTIONS)) {
+            /* if I am a server, add in my fns */
+            if (PMIX_PEER_IS_SERVER(pmix_globals.mypeer)) {
+                _get_fns(&kyresults, PMIX_SERVER_FUNCTIONS, &server_attrs);
+            } else {
+                PMIx_Argv_append_nosize(&cache, PMIX_SERVER_FUNCTIONS);
+            }
+
+        } else if (PMIX_CHECK_KEY(&query->qualifiers[m], PMIX_TOOL_ATTRIBUTES)) {
+            if (PMIX_PEER_IS_TOOL(pmix_globals.mypeer)) {
+                _get_attrs(&kyresults, &query->qualifiers[m], &tool_attrs);
+            }
+
+        } else if (PMIX_CHECK_KEY(&query->qualifiers[m], PMIX_TOOL_FUNCTIONS)) {
+            if (PMIX_PEER_IS_TOOL(pmix_globals.mypeer)) {
+                _get_fns(&kyresults, PMIX_TOOL_FUNCTIONS, &tool_attrs);
+            }
+
+        } else if (PMIX_CHECK_KEY(&query->qualifiers[m], PMIX_HOST_ATTRIBUTES)) {
+            /* if I am a server, add in the host's */
+            if (PMIX_PEER_IS_SERVER(pmix_globals.mypeer)) {
+                _get_attrs(&kyresults, &query->qualifiers[m], &host_attrs);
+            } else {
+                PMIx_Argv_append_nosize(&cache, PMIX_HOST_ATTRIBUTES);
+            }
+
+        } else if (PMIX_CHECK_KEY(&query->qualifiers[m], PMIX_HOST_FUNCTIONS)) {
+            /* if I am a server, add in the host's */
+            if (PMIX_PEER_IS_SERVER(pmix_globals.mypeer)) {
+                _get_fns(&kyresults, PMIX_HOST_FUNCTIONS, &host_attrs);
+            } else {
+                PMIx_Argv_append_nosize(&cache, PMIX_HOST_FUNCTIONS);
+            }
+        }
+
+        if (0 < (p = pmix_list_get_size(&kyresults))) {
+            PMIX_KVAL_NEW(kv, PMIX_QUERY_ATTRIBUTE_SUPPORT);
+            kv->value->type = PMIX_DATA_ARRAY;
+            /* create the data array to hold the results */
+            PMIX_DATA_ARRAY_CREATE(darray, p, PMIX_INFO);
+            kv->value->data.darray = darray;
+            iptr = (pmix_info_t *) darray->array;
+            p = 0;
+            PMIX_LIST_FOREACH (info, &kyresults, pmix_infolist_t) {
+                PMIX_INFO_XFER(&iptr[p], &info->info);
+                ++p;
+            }
+            pmix_list_append(&cd->results, &kv->super);
+        }
+        PMIX_LIST_DESTRUCT(&kyresults);
+    }
+
+request:
+    // if we don't need help then we are done
+    if (NULL == cache) {
+        if (NULL != cache) {
+            PMIx_Argv_free(cache);
+        }
+        return;
+    }
+
+    qry = PMIX_NEW(pmix_querylist_t);
+    PMIX_ARGV_APPEND(rc, qry->query.keys, PMIX_QUERY_ATTRIBUTE_SUPPORT);
+    if (PMIX_SUCCESS != rc) {
+        PMIX_ERROR_LOG(rc);
+        PMIX_RELEASE(qry);
+        return;
+    }
+    qry->query.nqual = PMIx_Argv_count(cache);
+    PMIX_INFO_CREATE(qry->query.qualifiers, qry->query.nqual);
+    for (n=0; NULL != cache[n]; n++) {
+        PMIX_INFO_LOAD(&qry->query.qualifiers[n], cache[n], NULL, PMIX_BOOL);
+    }
+    PMIx_Argv_free(cache);
+    pmix_list_append(unresolved, &qry->super);
+
+    return;
 }
 
 /*****   LOCATE A GIVEN ATTRIBUTE    *****/
 PMIX_EXPORT const char *pmix_attributes_lookup(const char *attr)
 {
-    size_t n;
+    pmix_keyindex_t *const kidx = &pmix_globals.keyindex;
 
-    for (n = 0; 0 != strlen(pmix_dictionary[n].name); n++) {
-        if (0 == strcasecmp(pmix_dictionary[n].name, attr)) {
-            return pmix_dictionary[n].string;
+    for (int i = 0; i < kidx->table->size; ++i) {
+        pmix_regattr_input_t *ra = pmix_pointer_array_get_item(kidx->table, i);
+        if (NULL == ra) break;
+        if (0 == strcasecmp(ra->name, attr)) {
+            return ra->string;
         }
     }
     return attr;
@@ -861,11 +812,13 @@ PMIX_EXPORT const char *pmix_attributes_lookup(const char *attr)
 
 PMIX_EXPORT const char *pmix_attributes_reverse_lookup(const char *attrstring)
 {
-    size_t n;
+    pmix_keyindex_t *const kidx = &pmix_globals.keyindex;
 
-    for (n = 0; 0 != strlen(pmix_dictionary[n].name); n++) {
-        if (0 == strcasecmp(pmix_dictionary[n].string, attrstring)) {
-            return pmix_dictionary[n].name;
+    for (int i = 0; i < kidx->table->size; ++i) {
+        pmix_regattr_input_t *ra = pmix_pointer_array_get_item(kidx->table, i);
+        if (NULL == ra) break;
+        if (0 == strcasecmp(ra->string, attrstring)) {
+            return ra->name;
         }
     }
     return attrstring;
@@ -873,11 +826,13 @@ PMIX_EXPORT const char *pmix_attributes_reverse_lookup(const char *attrstring)
 
 PMIX_EXPORT const pmix_regattr_input_t *pmix_attributes_lookup_term(char *attr)
 {
-    size_t n;
+    pmix_keyindex_t *const kidx = &pmix_globals.keyindex;
 
-    for (n = 0; 0 != strlen(pmix_dictionary[n].name); n++) {
-        if (0 == strcmp(pmix_dictionary[n].name, attr)) {
-            return &pmix_dictionary[n];
+    for (int i = 0; i < kidx->table->size; ++i) {
+        pmix_regattr_input_t *ra = pmix_pointer_array_get_item(kidx->table, i);
+        if (NULL == ra) break;
+        if (0 == strcasecmp(ra->name, attr)) {
+            return ra;
         }
     }
     return NULL;
@@ -896,23 +851,23 @@ PMIX_EXPORT char **pmix_attributes_print_functions(char *level)
 
     /* select title */
     if (0 == strcmp(level, PMIX_CLIENT_FUNCTIONS)) {
-        pmix_argv_append_nosize(&ans, title1);
+        PMIx_Argv_append_nosize(&ans, title1);
         lst = &client_attrs;
     } else if (0 == strcmp(level, PMIX_SERVER_FUNCTIONS)) {
-        pmix_argv_append_nosize(&ans, title2);
+        PMIx_Argv_append_nosize(&ans, title2);
         lst = &server_attrs;
     } else if (0 == strcmp(level, PMIX_HOST_FUNCTIONS)) {
-        pmix_argv_append_nosize(&ans, title3);
+        PMIx_Argv_append_nosize(&ans, title3);
         lst = &host_attrs;
     } else if (0 == strcmp(level, PMIX_TOOL_FUNCTIONS)) {
-        pmix_argv_append_nosize(&ans, title4);
+        PMIx_Argv_append_nosize(&ans, title4);
         lst = &tool_attrs;
     } else {
         return NULL;
     }
 
     PMIX_LIST_FOREACH (fnptr, lst, pmix_attribute_trk_t) {
-        pmix_argv_append_nosize(&ans, fnptr->function);
+        PMIx_Argv_append_nosize(&ans, fnptr->function);
     }
     return ans;
 }
@@ -924,7 +879,8 @@ PMIX_EXPORT char **pmix_attributes_print_functions(char *level)
 #define PMIX_PRINT_TYPE_COLUMN_WIDTH   20
 #define PMIX_PRINT_ATTR_COLUMN_WIDTH   141
 
-void pmix_attributes_print_attrs(char ***ans, char *function, pmix_regattr_t *attrs, size_t nattrs)
+void pmix_attributes_print_attrs(char ***ans, char *function,
+                                 pmix_regattr_t *attrs, size_t nattrs)
 {
     char line[PMIX_PRINT_ATTR_COLUMN_WIDTH], *tmp;
     size_t n, m, len;
@@ -938,7 +894,7 @@ void pmix_attributes_print_attrs(char ***ans, char *function, pmix_regattr_t *at
     }
     line[m++] = ':';
     line[m] = '\0';
-    pmix_argv_append_nosize(ans, line);
+    PMIx_Argv_append_nosize(ans, line);
 
     for (n = 0; n < nattrs; n++) {
         memset(line, ' ', PMIX_PRINT_ATTR_COLUMN_WIDTH);
@@ -949,6 +905,12 @@ void pmix_attributes_print_attrs(char ***ans, char *function, pmix_regattr_t *at
         }
         memcpy(line, attrs[n].name, len);
 
+        if (0 == strlen(attrs[n].string)) {
+            line[PMIX_PRINT_ATTR_COLUMN_WIDTH - 1] = '\0'; // ensure NULL termination
+            PMIx_Argv_append_nosize(ans, line);
+            continue;
+        }
+
         len = strlen(attrs[n].string);
         if (PMIX_PRINT_STRING_COLUMN_WIDTH < len) {
             len = PMIX_PRINT_STRING_COLUMN_WIDTH;
@@ -968,7 +930,7 @@ void pmix_attributes_print_attrs(char ***ans, char *function, pmix_regattr_t *at
                          + PMIX_PRINT_TYPE_COLUMN_WIDTH + 6],
                    attrs[n].description[m], len);
             line[PMIX_PRINT_ATTR_COLUMN_WIDTH - 1] = '\0'; // ensure NULL termination
-            pmix_argv_append_nosize(ans, line);
+            PMIx_Argv_append_nosize(ans, line);
             memset(line, ' ', PMIX_PRINT_ATTR_COLUMN_WIDTH);
             line[PMIX_PRINT_ATTR_COLUMN_WIDTH - 1] = '\0';
         }
@@ -986,13 +948,13 @@ void pmix_attributes_print_headers(char ***ans, char *level)
 
     /* select title */
     if (0 == strcmp(level, PMIX_CLIENT_ATTRIBUTES)) {
-        pmix_argv_append_nosize(ans, title1);
+        PMIx_Argv_append_nosize(ans, title1);
     } else if (0 == strcmp(level, PMIX_SERVER_ATTRIBUTES)) {
-        pmix_argv_append_nosize(ans, title2);
+        PMIx_Argv_append_nosize(ans, title2);
     } else if (0 == strcmp(level, PMIX_HOST_ATTRIBUTES)) {
-        pmix_argv_append_nosize(ans, title3);
+        PMIx_Argv_append_nosize(ans, title3);
     } else if (0 == strcmp(level, PMIX_TOOL_ATTRIBUTES)) {
-        pmix_argv_append_nosize(ans, title4);
+        PMIx_Argv_append_nosize(ans, title4);
     } else {
         return;
     }
@@ -1019,7 +981,7 @@ void pmix_attributes_print_headers(char ***ans, char *level)
     memcpy(&line[left], "DESCRIPTION", strlen("DESCRIPTION"));
     left += strlen("DESCRIPTION") + 1;
     line[left] = '\0';
-    pmix_argv_append_nosize(ans, line);
+    PMIx_Argv_append_nosize(ans, line);
 
     /* print the dashes under the column headers */
     memset(line, ' ', PMIX_PRINT_ATTR_COLUMN_WIDTH);
@@ -1044,7 +1006,7 @@ void pmix_attributes_print_headers(char ***ans, char *level)
         line[m] = '-';
         ++m;
     }
-    pmix_argv_append_nosize(ans, line);
+    PMIx_Argv_append_nosize(ans, line);
 }
 
 PMIX_EXPORT char **pmix_attributes_print_attr(char *level, char *function)
@@ -1072,40 +1034,52 @@ PMIX_EXPORT char **pmix_attributes_print_attr(char *level, char *function)
 
     /* print the column headers */
     pmix_attributes_print_headers(&ans, level);
-    memset(line, ' ', PMIX_PRINT_ATTR_COLUMN_WIDTH);
-    line[1] = '\0';
+
+    memset(line, '=', PMIX_PRINT_ATTR_COLUMN_WIDTH);
+    line[PMIX_PRINT_ATTR_COLUMN_WIDTH - 1] = '\0';
 
     /* can be comma-delimited list of functions */
-    tmp = pmix_argv_split(function, ',');
+    tmp = PMIx_Argv_split(function, ',');
     for (n = 0; NULL != tmp[n]; n++) {
         PMIX_LIST_FOREACH (fnptr, lst, pmix_attribute_trk_t) {
             if (0 == strcmp(tmp[n], "all") || 0 == strcmp(tmp[n], fnptr->function)) {
                 /* create an array of pmix_regattr_t for this function's attributes */
-                nattr = pmix_argv_count(fnptr->attrs);
-                PMIX_REGATTR_CREATE(rptr, nattr);
-                for (m = 0; m < nattr; m++) {
-                    rptr[m].name = strdup(fnptr->attrs[m]);
-                    PMIX_LOAD_KEY(rptr[m].string, pmix_attributes_lookup(fnptr->attrs[m]));
-                    dptr = pmix_attributes_lookup_term(fnptr->attrs[m]);
-                    if (NULL == dptr) {
-                        pmix_argv_free(tmp);
-                        pmix_argv_free(ans);
-                        PMIX_REGATTR_FREE(rptr, nattr);
-                        return NULL;
+                nattr = PMIx_Argv_count(fnptr->attrs);
+                if (0 == nattr || (1 == nattr &&
+                                   (0 == strcmp("N/A", fnptr->attrs[0]) ||
+                                    0 == strcmp("NONE", fnptr->attrs[0])))) {
+                    nattr = 1;
+                    PMIX_REGATTR_CREATE(rptr, nattr);
+                    rptr[0].name = strdup("NONE");
+                } else {
+                    PMIX_REGATTR_CREATE(rptr, nattr);
+                    for (m = 0; m < nattr; m++) {
+                        rptr[m].name = strdup(fnptr->attrs[m]);
+                        PMIX_LOAD_KEY(rptr[m].string, pmix_attributes_lookup(fnptr->attrs[m]));
+                        dptr = pmix_attributes_lookup_term(fnptr->attrs[m]);
+                        if (NULL == dptr) {
+                            PMIx_Argv_free(tmp);
+                            PMIx_Argv_free(ans);
+                            PMIX_REGATTR_FREE(rptr, nattr);
+                            return NULL;
+                        }
+                        rptr[m].type = dptr->type;
+                        rptr[m].description = PMIx_Argv_copy(dptr->description);
                     }
-                    rptr[m].type = dptr->type;
-                    rptr[m].description = pmix_argv_copy(dptr->description);
                 }
                 pmix_attributes_print_attrs(&ans, fnptr->function, rptr, nattr);
                 PMIX_REGATTR_FREE(rptr, nattr);
-                pmix_argv_append_nosize(&ans, line);
                 if (0 == strcmp(tmp[n], fnptr->function)) {
                     break;
                 }
+                /* add a spacer between functions */
+                PMIx_Argv_append_nosize(&ans, "   ");
+                PMIx_Argv_append_nosize(&ans, line);
+                PMIx_Argv_append_nosize(&ans, "   ");
             }
         }
     }
-    pmix_argv_free(tmp);
+    PMIx_Argv_free(tmp);
 
     return ans;
 }
