@@ -5,7 +5,8 @@
  *                         and Technology (RIST).  All rights reserved.
  * Copyright (c) 2017      IBM Corporation. All rights reserved.
  *
- * Copyright (c) 2021-2022 Nanook Consulting.  All rights reserved.
+ * Copyright (c) 2021-2023 Nanook Consulting.  All rights reserved.
+ * Copyright (c) 2023      Triad National Security, LLC. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -14,7 +15,7 @@
  */
 #include "src/include/pmix_config.h"
 
-#include "pmix.h"
+#include "include/pmix.h"
 #include "pmix_common.h"
 #include "include/pmix_server.h"
 
@@ -168,6 +169,7 @@ pmix_status_t pmix_notify_server_of_event(pmix_status_t status, const pmix_proc_
     pmix_cb_t *cb;
     pmix_event_chain_t *chain = NULL;
     size_t n;
+    bool holdcd;
     pmix_notify_caddy_t *cd;
 
     pmix_output_verbose(2, pmix_client_globals.event_output,
@@ -177,6 +179,19 @@ pmix_status_t pmix_notify_server_of_event(pmix_status_t status, const pmix_proc_
                         pmix_client_globals.myserver->info->pname.rank, PMIx_Error_string(status),
                         PMIx_Data_range_string(range));
 
+    holdcd = true;
+    if (0 < ninfo) {
+        /* check for caching instructions */
+        for (n = 0; n < ninfo; n++) {
+            if (PMIX_CHECK_KEY(&info[n], PMIX_EVENT_DO_NOT_CACHE)) {
+                if (PMIX_INFO_TRUE(&info[n])) {
+                    holdcd = false;
+                }
+                break;
+            }
+        }
+    }
+
     if (PMIX_RANGE_PROC_LOCAL != range) {
         /* create the msg object */
         msg = PMIX_NEW(pmix_buffer_t);
@@ -234,44 +249,45 @@ pmix_status_t pmix_notify_server_of_event(pmix_status_t status, const pmix_proc_
         /* prep the chain for processing */
         pmix_prep_event_chain(chain, info, ninfo, true);
 
-        /* we need to cache this event so we can pass it into
-         * ourselves should someone later register for it */
-        cd = PMIX_NEW(pmix_notify_caddy_t);
-        cd->status = status;
-        PMIX_LOAD_PROCID(&cd->source, chain->source.nspace, chain->source.rank);
-        cd->range = chain->range;
-        if (0 < chain->ninfo) {
-            cd->ninfo = chain->ninfo;
-            PMIX_INFO_CREATE(cd->info, cd->ninfo);
-            cd->nondefault = chain->nondefault;
-            /* need to copy the info */
-            for (n = 0; n < cd->ninfo; n++) {
-                PMIX_INFO_XFER(&cd->info[n], &chain->info[n]);
+        if (PMIX_RANGE_PROC_LOCAL == range && holdcd) {
+            /* we need to cache this event so we can pass it into
+             * ourselves should someone later register for it */
+            cd = PMIX_NEW(pmix_notify_caddy_t);
+            cd->status = status;
+            PMIX_LOAD_PROCID(&cd->source, chain->source.nspace, chain->source.rank);
+            cd->range = chain->range;
+            if (0 < chain->ninfo) {
+                cd->ninfo = chain->ninfo;
+                PMIX_INFO_CREATE(cd->info, cd->ninfo);
+                cd->nondefault = chain->nondefault;
+                /* need to copy the info */
+                for (n = 0; n < cd->ninfo; n++) {
+                    PMIX_INFO_XFER(&cd->info[n], &chain->info[n]);
+                }
             }
-        }
-        if (NULL != chain->targets) {
-            cd->ntargets = chain->ntargets;
-            PMIX_PROC_CREATE(cd->targets, cd->ntargets);
-            memcpy(cd->targets, chain->targets, cd->ntargets * sizeof(pmix_proc_t));
-        }
-        if (NULL != chain->affected) {
-            cd->naffected = chain->naffected;
-            PMIX_PROC_CREATE(cd->affected, cd->naffected);
-            if (NULL == cd->affected) {
-                cd->naffected = 0;
-                rc = PMIX_ERR_NOMEM;
+            if (NULL != chain->targets) {
+                cd->ntargets = chain->ntargets;
+                PMIX_PROC_CREATE(cd->targets, cd->ntargets);
+                memcpy(cd->targets, chain->targets, cd->ntargets * sizeof(pmix_proc_t));
+            }
+            if (NULL != chain->affected) {
+                cd->naffected = chain->naffected;
+                PMIX_PROC_CREATE(cd->affected, cd->naffected);
+                if (NULL == cd->affected) {
+                    cd->naffected = 0;
+                    rc = PMIX_ERR_NOMEM;
+                    goto cleanup;
+                }
+                memcpy(cd->affected, chain->affected, cd->naffected * sizeof(pmix_proc_t));
+            }
+            rc = pmix_notify_event_cache(cd);
+            if (PMIX_SUCCESS != rc) {
+                PMIX_ERROR_LOG(rc);
+                PMIX_RELEASE(cd);
                 goto cleanup;
             }
-            memcpy(cd->affected, chain->affected, cd->naffected * sizeof(pmix_proc_t));
+            chain->cached = true;
         }
-        /* cache it */
-        rc = pmix_notify_event_cache(cd);
-        if (PMIX_SUCCESS != rc) {
-            PMIX_ERROR_LOG(rc);
-            PMIX_RELEASE(cd);
-            goto cleanup;
-        }
-        chain->cached = true;
     }
 
     if (PMIX_RANGE_PROC_LOCAL != range && NULL != msg) {
@@ -367,7 +383,7 @@ static void cycle_events(int sd, short args, void *cbdata)
     }
 
     /* save this handler's returned status */
-    if (NULL != chain->evhdlr->name) {
+    if (NULL != chain->evhdlr && NULL != chain->evhdlr->name) {
         pmix_strncpy(newinfo[cnt].key, chain->evhdlr->name, PMIX_MAX_KEYLEN);
     } else {
         pmix_strncpy(newinfo[cnt].key, "UNKNOWN", PMIX_MAX_KEYLEN);
@@ -1040,49 +1056,6 @@ static void _notify_client_event(int sd, short args, void *cbdata)
         return;
     }
 
-    /* check to see if this is a group_complete notification
-     * indicating that a group has asynchronously been formed.
-     * If it is, then we need to track the group */
-    if (PMIX_GROUP_CONSTRUCT_COMPLETE == cd->status) {
-        char *grpid = NULL;
-        pmix_group_t *grp;
-        /* must include the group id */
-        for (n = 0; n < cd->ninfo; n++) {
-            if (PMIX_CHECK_KEY(&cd->info[n], PMIX_GROUP_ID)) {
-                grpid = cd->info[n].value.data.string;
-                break;
-            }
-        }
-        if (NULL == grpid) {
-            /* failed to provide the ID */
-            PMIX_ERROR_LOG(PMIX_ERR_BAD_PARAM);
-            /* notify the caller */
-            if (NULL != cd->cbfunc) {
-                cd->cbfunc(PMIX_ERR_BAD_PARAM, cd->cbdata);
-            }
-            PMIX_RELEASE(cd);
-            PMIX_RELEASE(chain);
-            return;
-        }
-        /* must include members */
-        if (NULL == cd->targets || 0 == cd->ntargets) {
-            PMIX_ERROR_LOG(PMIX_ERR_BAD_PARAM);
-            /* notify the caller */
-            if (NULL != cd->cbfunc) {
-                cd->cbfunc(PMIX_ERR_BAD_PARAM, cd->cbdata);
-            }
-            PMIX_RELEASE(cd);
-            PMIX_RELEASE(chain);
-            return;
-        }
-        grp = PMIX_NEW(pmix_group_t);
-        grp->grpid = strdup(grpid);
-        grp->nmbrs = cd->ntargets;
-        PMIX_PROC_CREATE(grp->members, grp->nmbrs);
-        memcpy(grp->members, cd->targets, cd->ntargets * sizeof(pmix_proc_t));
-        pmix_list_append(&pmix_server_globals.groups, &grp->super);
-    }
-
     holdcd = false;
     if (PMIX_RANGE_PROC_LOCAL != cd->range) {
         PMIX_CONSTRUCT(&trk, pmix_list_t);
@@ -1097,11 +1070,11 @@ static void _notify_client_event(int sd, short args, void *cbdata)
                     /* if this client was the source of the event, then
                      * don't send it back as they will have processed it
                      * when they generated it */
-                    if (PMIX_CHECK_PROCID(&cd->source, &pr->peer->info->pname)) {
+                    if (PMIX_CHECK_NAMES(&cd->source, &pr->peer->info->pname)) {
                         continue;
                     }
                     /* don't notify ourselves - we handle this internally */
-                    if (PMIX_CHECK_PROCID(&pmix_globals.myid, &pr->peer->info->pname)) {
+                    if (PMIX_CHECK_NAMES(&pmix_globals.myid, &pr->peer->info->pname)) {
                         continue;
                     }
                     /* if we have already notified this client, then don't do it again */
