@@ -15,7 +15,7 @@
  *                         and Technology (RIST).  All rights reserved.
  * Copyright (c) 2019      Mellanox Technologies, Inc.
  *                         All rights reserved.
- * Copyright (c) 2021-2023 Nanook Consulting.  All rights reserved.
+ * Copyright (c) 2021-2025 Nanook Consulting  All rights reserved.
  * Copyright (c) 2023      Triad National Security, LLC. All rights reserved.
  * $COPYRIGHT$
  *
@@ -104,6 +104,38 @@ typedef struct {
     pmix_data_type_t type;
     char **description;
 } pmix_regattr_input_t;
+#define PMIX_REGATTR_INPUT_NEW(a, i, n, s, t, d)                            \
+do {                                                                        \
+    (a) = (pmix_regattr_input_t*)pmix_malloc(sizeof(pmix_regattr_input_t)); \
+    if (NULL != (a)) {                                                      \
+        memset((a), 0, sizeof(pmix_regattr_input_t));                       \
+        (a)->index = (i);                                                   \
+        if (NULL != (n)) {                                                    \
+            (a)->name = strdup((n));                                       \
+        }                                                                   \
+        if (NULL != (s)) {                                                    \
+            (a)->string = strdup((s));                                        \
+        }                                                                   \
+        (a)->type = (t);                                                    \
+        if (NULL != (d)) {                                                    \
+            (a)->description = PMIx_Argv_copy((d));                           \
+        }                                                                   \
+    }                                                                       \
+} while(0)
+#define PMIX_REGATTR_INPUT_FREE(a)      \
+do {                                    \
+    if (NULL != (a)) {                  \
+        if (NULL != (a)->name) {        \
+            free((a)->name);            \
+        }                               \
+        if (NULL != (a)->string)        \
+            free((a)->string);          \
+        }                               \
+        if (NULL != (a)->description) { \
+            free((a)->description);     \
+        }                               \
+    }                                   \
+} while(0)
 
 /* define a struct for holding entries in the
  * dictionary of event strings */
@@ -208,6 +240,9 @@ typedef uint8_t pmix_cmd_t;
 #define PMIX_FABRIC_UPDATE_CMD            31
 #define PMIX_COMPUTE_DEVICE_DISTANCES_CMD 32
 #define PMIX_REFRESH_CACHE                33
+#define PMIX_RESOLVE_PEERS_CMD            37
+#define PMIX_RESOLVE_NODE_CMD             38
+
 
 /* provide a "pretty-print" function for cmds */
 const char *pmix_command_string(pmix_cmd_t cmd);
@@ -430,6 +465,7 @@ typedef struct {
     size_t remote_id;
     pmix_proc_t *procs;
     size_t nprocs;
+    pmix_iof_flags_t flags;
     pmix_iof_channel_t channels;
     pmix_iof_cbfunc_t cbfunc;
     pmix_hdlr_reg_cbfunc_t regcbfunc;
@@ -437,8 +473,6 @@ typedef struct {
 } pmix_iof_req_t;
 PMIX_CLASS_DECLARATION(pmix_iof_req_t);
 
-typedef void (*pmix_pstrg_query_cbfunc_t)(pmix_status_t status, pmix_list_t *results, void *cbdata);
-
 /* caddy for query requests */
 typedef struct {
     pmix_object_t super;
@@ -463,7 +497,6 @@ typedef struct {
     pmix_release_cbfunc_t relcbfunc;
     pmix_credential_cbfunc_t credcbfunc;
     pmix_validation_cbfunc_t validcbfunc;
-    pmix_pstrg_query_cbfunc_t stqcbfunc;
     void *cbdata;
 } pmix_query_caddy_t;
 PMIX_CLASS_DECLARATION(pmix_query_caddy_t);
@@ -508,6 +541,7 @@ typedef struct {
     pmix_info_t *info;      // array of info structs
     size_t ninfo;           // number of info structs in array
     pmix_list_t grpinfo;    // list of group info to be distributed
+    int grpop;              // the group operation being tracked
     pmix_collect_t collect_type; // whether or not data is to be returned at completion
     pmix_modex_cbfunc_t modexcbfunc;
     pmix_op_cbfunc_t op_cbfunc;
@@ -528,6 +562,7 @@ typedef struct {
     pmix_peer_t *peer;
     pmix_info_t *info;
     size_t ninfo;
+    pmix_query_t *query;
 } pmix_server_caddy_t;
 PMIX_CLASS_DECLARATION(pmix_server_caddy_t);
 
@@ -697,6 +732,22 @@ typedef struct {
 } pmix_notify_caddy_t;
 PMIX_CLASS_DECLARATION(pmix_notify_caddy_t);
 
+typedef struct {
+    pmix_object_t super;
+    /** Points to key <--> index translation table. */
+    pmix_pointer_array_t *table;
+    /** Stores the next ID. */
+    uint32_t next_id;
+} pmix_keyindex_t;
+PMIX_CLASS_DECLARATION(pmix_keyindex_t);
+
+#define PMIX_KEYINDEX_STATIC_INIT                 \
+{                                                 \
+    .super = PMIX_OBJ_STATIC_INIT(pmix_object_t), \
+    .table = NULL,                                \
+    .next_id = PMIX_INDEX_BOUNDARY                \
+}
+
 /****    GLOBAL STORAGE    ****/
 /* define a global construct that includes values that must be shared
  * between various parts of the code library. The client, tool,
@@ -740,8 +791,7 @@ typedef struct {
     bool external_topology;
     bool external_progress;
     pmix_iof_flags_t iof_flags;
-    pmix_pointer_array_t keyindex;  // translation table of key <-> index
-    uint32_t next_keyid;
+    pmix_keyindex_t keyindex;
 } pmix_globals_t;
 
 /* provide access to a function to cleanup epilogs */
