@@ -17,7 +17,7 @@
  * Copyright (c) 2014-2015 Los Alamos National Security, LLC. All rights
  *                         reserved.
  * Copyright (c) 2016-2020 Intel, Inc.  All rights reserved.
- * Copyright (c) 2021      Nanook Consulting.  All rights reserved.
+ * Copyright (c) 2021-2022 Nanook Consulting.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -47,18 +47,18 @@
 #    include <netdb.h>
 #endif
 
-#include "include/pmix_common.h"
+#include "pmix_common.h"
 #include "src/class/pmix_list.h"
 #include "src/include/pmix_globals.h"
-#include "src/mca/base/base.h"
+#include "src/mca/base/pmix_base.h"
 #include "src/mca/base/pmix_mca_base_component_repository.h"
 #include "src/mca/mca.h"
 #include "src/mca/pdl/base/base.h"
 #include "src/mca/pinstalldirs/pinstalldirs.h"
-#include "src/util/argv.h"
-#include "src/util/output.h"
+#include "src/util/pmix_argv.h"
+#include "src/util/pmix_output.h"
 #include "src/util/pmix_environ.h"
-#include "src/util/show_help.h"
+#include "src/util/pmix_show_help.h"
 
 #if PMIX_HAVE_PDL_SUPPORT
 /*
@@ -78,8 +78,6 @@ static int component_find_check(pmix_mca_base_framework_t *framework,
 struct pmix_mca_base_open_only_dummy_component_t {
     /** MCA base component */
     pmix_mca_base_component_t version;
-    /** MCA base data */
-    pmix_mca_base_component_data_t data;
 };
 typedef struct pmix_mca_base_open_only_dummy_component_t pmix_mca_base_open_only_dummy_component_t;
 
@@ -108,7 +106,8 @@ int pmix_mca_base_component_find(const char *directory, pmix_mca_base_framework_
     PMIX_HIDE_UNUSED_PARAMS(open_dso_components);
 
     pmix_output_verbose(PMIX_MCA_BASE_VERBOSE_COMPONENT, framework->framework_output,
-                        "mca: base: component_find: searching %s for %s components", directory,
+                        "mca: base: component_find: searching %s for %s components",
+                        (NULL == directory) ? "NULL" : directory,
                         framework->framework_name);
 
     if (!ignore_requested) {
@@ -141,10 +140,9 @@ int pmix_mca_base_component_find(const char *directory, pmix_mca_base_framework_
         find_dyn_components(directory, framework, (const char **) requested_component_names,
                             include_mode);
     } else {
-        pmix_output_verbose(
-            PMIX_MCA_BASE_VERBOSE_INFO, 0,
-            "pmix:mca: base: component_find: dso loading for %s MCA components disabled",
-            framework->framework_name);
+        pmix_output_verbose(PMIX_MCA_BASE_VERBOSE_INFO, 0,
+                            "pmix:mca: base: component_find: dso loading for %s MCA components disabled",
+                            framework->framework_name);
     }
 #endif
 
@@ -157,7 +155,7 @@ int pmix_mca_base_component_find(const char *directory, pmix_mca_base_framework_
 component_find_out:
 
     if (NULL != requested_component_names) {
-        pmix_argv_free(requested_component_names);
+        PMIx_Argv_free(requested_component_names);
     }
 
     /* All done */
@@ -170,7 +168,7 @@ int pmix_mca_base_component_find_finalize(void)
     return PMIX_SUCCESS;
 }
 
-int pmix_mca_base_components_filter(pmix_mca_base_framework_t *framework, uint32_t filter_flags)
+int pmix_mca_base_components_filter(pmix_mca_base_framework_t *framework)
 {
     pmix_list_t *components = &framework->framework_components;
     int output_id = framework->framework_output;
@@ -181,7 +179,7 @@ int pmix_mca_base_components_filter(pmix_mca_base_framework_t *framework, uint32
 
     assert(NULL != components);
 
-    if (0 == filter_flags && NULL == framework->framework_selection) {
+    if (NULL == framework->framework_selection) {
         return PMIX_SUCCESS;
     }
 
@@ -193,31 +191,16 @@ int pmix_mca_base_components_filter(pmix_mca_base_framework_t *framework, uint32
 
     PMIX_LIST_FOREACH_SAFE (cli, next, components, pmix_mca_base_component_list_item_t) {
         const pmix_mca_base_component_t *component = cli->cli_component;
-        pmix_mca_base_open_only_dummy_component_t *dummy
-            = (pmix_mca_base_open_only_dummy_component_t *) cli->cli_component;
 
         can_use = use_component(include_mode, (const char **) requested_component_names,
                                 cli->cli_component->pmix_mca_component_name);
 
-        if (!can_use || (filter_flags & dummy->data.param_field) != filter_flags) {
-            if (can_use && (filter_flags & PMIX_MCA_BASE_METADATA_PARAM_CHECKPOINT)
-                && !(PMIX_MCA_BASE_METADATA_PARAM_CHECKPOINT & dummy->data.param_field)) {
-                pmix_output_verbose(PMIX_MCA_BASE_VERBOSE_COMPONENT, output_id,
-                                    "pmix:mca: base: components_filter: "
-                                    "(%s) Component %s is *NOT* Checkpointable - Disabled",
-                                    component->reserved, component->pmix_mca_component_name);
-            }
-
+        if (!can_use) {
             pmix_list_remove_item(components, &cli->super);
 
             pmix_mca_base_component_unload(component, output_id);
 
             PMIX_RELEASE(cli);
-        } else if (filter_flags & PMIX_MCA_BASE_METADATA_PARAM_CHECKPOINT) {
-            pmix_output_verbose(PMIX_MCA_BASE_VERBOSE_COMPONENT, output_id,
-                                "pmix:mca: base: components_filter: "
-                                "(%s) Component %s is Checkpointable",
-                                component->reserved, component->pmix_mca_component_name);
         }
     }
 
@@ -228,7 +211,7 @@ int pmix_mca_base_components_filter(pmix_mca_base_framework_t *framework, uint32
     }
 
     if (NULL != requested_component_names) {
-        pmix_argv_free(requested_component_names);
+        PMIx_Argv_free(requested_component_names);
     }
 
     return ret;
@@ -241,7 +224,7 @@ int pmix_mca_base_components_filter(pmix_mca_base_framework_t *framework, uint32
  * the specified type (and possibly of a given name).
  *
  * Note that we use our own path iteration functionality because we
- * need to look at companion .ompi_info files in the same directory as
+ * need to look at companion .pmix_info files in the same directory as
  * the library to generate dependencies, etc.
  */
 static void find_dyn_components(const char *path, pmix_mca_base_framework_t *framework,
@@ -252,11 +235,12 @@ static void find_dyn_components(const char *path, pmix_mca_base_framework_t *fra
     int ret;
 
     pmix_output_verbose(PMIX_MCA_BASE_VERBOSE_COMPONENT, framework->framework_output,
-                        "mca: base: find_dyn_components: checking %s for %s components", path,
+                        "mca: base: find_dyn_components: checking %s for %s components",
+                        (NULL == path) ? "NULL" : path,
                         framework->framework_name);
 
     if (NULL != path) {
-        ret = pmix_mca_base_component_repository_add(path);
+        ret = pmix_mca_base_component_repository_add(framework->framework_project, path);
         if (PMIX_SUCCESS != ret) {
             return;
         }
@@ -344,6 +328,9 @@ static int component_find_check(pmix_mca_base_framework_t *framework,
             gethostname(h, sizeof(h) - 1);
             pmix_show_help("help-pmix-mca-base.txt", "find-available:not-valid", true, h,
                            framework->framework_name, requested_component_names[i]);
+            if (pmix_mca_base_component_abort_on_load_error) {
+                return PMIX_ERR_NOT_FOUND;
+            }
         }
     }
 
@@ -380,7 +367,7 @@ int pmix_mca_base_component_parse_requested(const char *requested, bool *include
     }
 
     /* Split up the value into individual component names */
-    *requested_component_names = pmix_argv_split(requested, ',');
+    *requested_component_names = PMIx_Argv_split(requested, ',');
 
     /* All done */
     return PMIX_SUCCESS;
