@@ -13,7 +13,7 @@
  * Copyright (c) 2015      Los Alamos National Security, LLC. All rights
  *                         reserved.
  * Copyright (c) 2016-2020 Intel, Inc.  All rights reserved.
- * Copyright (c) 2021      Nanook Consulting.  All rights reserved.
+ * Copyright (c) 2021-2022 Nanook Consulting.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -28,11 +28,11 @@
  */
 
 #include "src/include/pmix_config.h"
-#include "include/pmix_common.h"
+#include "pmix_common.h"
 
 #include "src/hwloc/pmix_hwloc.h"
 #include "src/mca/pnet/base/base.h"
-#include "src/util/argv.h"
+#include "src/util/pmix_argv.h"
 
 #include "pnet_opa.h"
 
@@ -45,29 +45,22 @@ static pmix_status_t component_register(void);
  * Instantiate the public struct with all of our public information
  * and pointers to our public functions in it
  */
-pmix_pnet_opa_component_t mca_pnet_opa_component = {
+pmix_pnet_opa_component_t pmix_mca_pnet_opa_component = {
     .super = {
-        .base = {
-            PMIX_PNET_BASE_VERSION_1_0_0,
+        PMIX_PNET_BASE_VERSION_1_0_0,
 
-            /* Component name and version */
-            .pmix_mca_component_name = "opa",
-            PMIX_MCA_BASE_MAKE_VERSION(component,
-                                       PMIX_MAJOR_VERSION,
-                                       PMIX_MINOR_VERSION,
-                                       PMIX_RELEASE_VERSION),
+        /* Component name and version */
+        .pmix_mca_component_name = "opa",
+        PMIX_MCA_BASE_MAKE_VERSION(component,
+                                   PMIX_MAJOR_VERSION,
+                                   PMIX_MINOR_VERSION,
+                                   PMIX_RELEASE_VERSION),
 
-            /* Component open and close functions */
-            .pmix_mca_open_component = component_open,
-            .pmix_mca_close_component = component_close,
-            .pmix_mca_register_component_params = component_register,
-            .pmix_mca_query_component = component_query,
-        },
-        .data = {
-            /* The component is checkpoint ready */
-            PMIX_MCA_BASE_METADATA_PARAM_CHECKPOINT,
-            .reserved = {0}
-        }
+        /* Component open and close functions */
+        .pmix_mca_open_component = component_open,
+        .pmix_mca_close_component = component_close,
+        .pmix_mca_register_component_params = component_register,
+        .pmix_mca_query_component = component_query,
     },
     .include = NULL,
     .exclude = NULL
@@ -75,26 +68,26 @@ pmix_pnet_opa_component_t mca_pnet_opa_component = {
 
 static pmix_status_t component_register(void)
 {
-    pmix_mca_base_component_t *component = &mca_pnet_opa_component.super.base;
+    pmix_mca_base_component_t *component = &pmix_mca_pnet_opa_component.super;
 
-    mca_pnet_opa_component.incparms = "HFI_*,PSM2_*";
+    pmix_mca_pnet_opa_component.incparms = "HFI_*,PSM2_*";
     (void) pmix_mca_base_component_var_register(
         component, "include_envars",
         "Comma-delimited list of envars to harvest (\'*\' and \'?\' supported)",
-        PMIX_MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0, PMIX_INFO_LVL_2, PMIX_MCA_BASE_VAR_SCOPE_LOCAL,
-        &mca_pnet_opa_component.incparms);
-    if (NULL != mca_pnet_opa_component.incparms) {
-        mca_pnet_opa_component.include = pmix_argv_split(mca_pnet_opa_component.incparms, ',');
+        PMIX_MCA_BASE_VAR_TYPE_STRING,
+        &pmix_mca_pnet_opa_component.incparms);
+    if (NULL != pmix_mca_pnet_opa_component.incparms) {
+        pmix_mca_pnet_opa_component.include = PMIx_Argv_split(pmix_mca_pnet_opa_component.incparms, ',');
     }
 
-    mca_pnet_opa_component.excparms = NULL;
+    pmix_mca_pnet_opa_component.excparms = NULL;
     (void) pmix_mca_base_component_var_register(
         component, "exclude_envars",
         "Comma-delimited list of envars to exclude (\'*\' and \'?\' supported)",
-        PMIX_MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0, PMIX_INFO_LVL_2, PMIX_MCA_BASE_VAR_SCOPE_LOCAL,
-        &mca_pnet_opa_component.excparms);
-    if (NULL != mca_pnet_opa_component.excparms) {
-        mca_pnet_opa_component.exclude = pmix_argv_split(mca_pnet_opa_component.excparms, ',');
+        PMIX_MCA_BASE_VAR_TYPE_STRING,
+        &pmix_mca_pnet_opa_component.excparms);
+    if (NULL != pmix_mca_pnet_opa_component.excparms) {
+        pmix_mca_pnet_opa_component.exclude = PMIx_Argv_split(pmix_mca_pnet_opa_component.excparms, ',');
     }
 
     return PMIX_SUCCESS;
