@@ -1,7 +1,7 @@
 /*
  * Copyright (c) 2019      IBM Corporation.  All rights reserved.
  * Copyright (c) 2020      Intel, Inc.  All rights reserved.
- * Copyright (c) 2021      Nanook Consulting.  All rights reserved.
+ * Copyright (c) 2021-2022 Nanook Consulting.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -19,7 +19,7 @@
 BEGIN_C_DECLS
 
 /* the component must be visible data for the linker to find it */
-PMIX_EXPORT extern pmix_psquash_base_component_t mca_psquash_flex128_component;
+PMIX_EXPORT extern pmix_psquash_base_component_t pmix_mca_psquash_flex128_component;
 extern pmix_psquash_base_module_t pmix_flex128_module;
 
 END_C_DECLS
