@@ -14,7 +14,7 @@
  * Copyright (c) 2014-2020 Intel, Inc.  All rights reserved.
  * Copyright (c) 2015-2020 Research Organization for Information Science
  *                         and Technology (RIST).  All rights reserved.
- * Copyright (c) 2021-2023 Nanook Consulting.  All rights reserved.
+ * Copyright (c) 2021-2024 Nanook Consulting  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -45,8 +45,10 @@
 #include "src/mca/base/pmix_mca_base_var.h"
 #include "src/mca/mca.h"
 #include "src/server/pmix_server_ops.h"
+#include "src/util/pmix_basename.h"
 #include "src/util/pmix_error.h"
 #include "src/util/pmix_os_dirpath.h"
+#include "src/util/pmix_os_path.h"
 #include "src/util/pmix_environ.h"
 #include "src/util/pmix_show_help.h"
 
@@ -76,15 +78,18 @@ pmix_ptl_base_t pmix_ptl_base = {
     .report_uri = NULL,
     .uri = NULL,
     .urifile = NULL,
+    .sysctrlr_filename = NULL,
     .scheduler_filename = NULL,
     .system_filename = NULL,
     .session_filename = NULL,
     .nspace_filename = NULL,
     .pid_filename = NULL,
     .rendezvous_filename = NULL,
+    .created_rendezvous_dir = false,
     .created_rendezvous_file = false,
     .created_session_tmpdir = false,
     .created_system_tmpdir = false,
+    .created_sysctrlr_filename = false,
     .created_scheduler_filename = false,
     .created_system_filename = false,
     .created_session_filename = false,
@@ -132,23 +137,21 @@ static int pmix_ptl_register(pmix_mca_base_register_flag_t flags)
                                &max_msg_size);
     pmix_ptl_base.max_msg_size = max_msg_size * 1024 * 1024;
 
-    idx = pmix_mca_base_var_register(
-        "pmix", "ptl", "base", "if_include",
-        "Comma-delimited list of devices and/or CIDR notation of TCP networks "
-        "(e.g., \"eth0,192.168.0.0/16\").  Mutually exclusive with ptl_tcp_if_exclude.",
-        PMIX_MCA_BASE_VAR_TYPE_STRING,
-        &pmix_ptl_base.if_include);
+    idx = pmix_mca_base_var_register("pmix", "ptl", "base", "if_include",
+                                     "Comma-delimited list of devices and/or CIDR notation of TCP networks "
+                                     "(e.g., \"eth0,192.168.0.0/16\").  Mutually exclusive with ptl_tcp_if_exclude.",
+                                     PMIX_MCA_BASE_VAR_TYPE_STRING,
+                                     &pmix_ptl_base.if_include);
     (void) pmix_mca_base_var_register_synonym(idx, "pmix", "ptl", "tcp", "if_include",
                                               PMIX_MCA_BASE_VAR_SYN_FLAG_DEPRECATED);
 
-    idx = pmix_mca_base_var_register(
-        "pmix", "ptl", "base", "if_exclude",
-        "Comma-delimited list of devices and/or CIDR notation of TCP networks to NOT use "
-        "-- all devices not matching these specifications will be used (e.g., "
-        "\"eth0,192.168.0.0/16\"). "
-        "If set to a non-default value, it is mutually exclusive with ptl_tcp_if_include.",
-        PMIX_MCA_BASE_VAR_TYPE_STRING,
-        &pmix_ptl_base.if_exclude);
+    idx = pmix_mca_base_var_register("pmix", "ptl", "base", "if_exclude",
+                                     "Comma-delimited list of devices and/or CIDR notation of TCP networks to NOT use "
+                                     "-- all devices not matching these specifications will be used (e.g., "
+                                     "\"eth0,192.168.0.0/16\"). "
+                                     "If set to a non-default value, it is mutually exclusive with ptl_tcp_if_include.",
+                                     PMIX_MCA_BASE_VAR_TYPE_STRING,
+                                     &pmix_ptl_base.if_exclude);
     (void) pmix_mca_base_var_register_synonym(idx, "pmix", "ptl", "tcp", "if_exclude",
                                               PMIX_MCA_BASE_VAR_SYN_FLAG_DEPRECATED);
 
@@ -185,35 +188,31 @@ static int pmix_ptl_register(pmix_mca_base_register_flag_t flags)
     (void) pmix_mca_base_var_register_synonym(idx, "pmix", "ptl", "tcp", "disable_ipv6_family",
                                               PMIX_MCA_BASE_VAR_SYN_FLAG_DEPRECATED);
 
-    idx = pmix_mca_base_var_register(
-        "pmix", "ptl", "base", "connection_wait_time",
-        "Number of seconds to wait for the server connection file to appear",
-        PMIX_MCA_BASE_VAR_TYPE_INT,
-        &pmix_ptl_base.wait_to_connect);
+    idx = pmix_mca_base_var_register("pmix", "ptl", "base", "connection_wait_time",
+                                     "Number of seconds to wait for the server connection file to appear",
+                                     PMIX_MCA_BASE_VAR_TYPE_INT,
+                                     &pmix_ptl_base.wait_to_connect);
     (void) pmix_mca_base_var_register_synonym(idx, "pmix", "ptl", "tcp", "connection_wait_time",
                                               PMIX_MCA_BASE_VAR_SYN_FLAG_DEPRECATED);
 
-    idx = pmix_mca_base_var_register(
-        "pmix", "ptl", "base", "max_retries",
-        "Number of times to look for the connection file before quitting",
-        PMIX_MCA_BASE_VAR_TYPE_INT,
-        &pmix_ptl_base.max_retries);
+    idx = pmix_mca_base_var_register("pmix", "ptl", "base", "max_retries",
+                                     "Number of times to look for the connection file before quitting",
+                                     PMIX_MCA_BASE_VAR_TYPE_INT,
+                                     &pmix_ptl_base.max_retries);
     (void) pmix_mca_base_var_register_synonym(idx, "pmix", "ptl", "tcp", "max_retries",
                                               PMIX_MCA_BASE_VAR_SYN_FLAG_DEPRECATED);
 
-    idx = pmix_mca_base_var_register(
-        "pmix", "ptl", "base", "handshake_wait_time",
-        "Number of seconds to wait for the server reply to the handshake request",
-        PMIX_MCA_BASE_VAR_TYPE_INT,
-        &pmix_ptl_base.handshake_wait_time);
+    idx = pmix_mca_base_var_register("pmix", "ptl", "base", "handshake_wait_time",
+                                     "Number of seconds to wait for the server reply to the handshake request",
+                                     PMIX_MCA_BASE_VAR_TYPE_INT,
+                                     &pmix_ptl_base.handshake_wait_time);
     (void) pmix_mca_base_var_register_synonym(idx, "pmix", "ptl", "tcp", "handshake_wait_time",
                                               PMIX_MCA_BASE_VAR_SYN_FLAG_DEPRECATED);
 
-    idx = pmix_mca_base_var_register(
-        "pmix", "ptl", "base", "handshake_max_retries",
-        "Number of times to retry the handshake request before giving up",
-        PMIX_MCA_BASE_VAR_TYPE_INT,
-        &pmix_ptl_base.handshake_max_retries);
+    idx = pmix_mca_base_var_register("pmix", "ptl", "base", "handshake_max_retries",
+                                     "Number of times to retry the handshake request before giving up",
+                                     PMIX_MCA_BASE_VAR_TYPE_INT,
+                                     &pmix_ptl_base.handshake_max_retries);
     (void) pmix_mca_base_var_register_synonym(idx, "pmix", "ptl", "tcp", "handshake_max_retries",
                                               PMIX_MCA_BASE_VAR_SYN_FLAG_DEPRECATED);
 
@@ -227,9 +226,33 @@ static int pmix_ptl_register(pmix_mca_base_register_flag_t flags)
     return PMIX_SUCCESS;
 }
 
+static bool _check_file(const char *root, const char *path)
+{
+    struct stat st;
+    char *fullpath;
+
+    /*
+     * Keep:
+     *  - non-zero files starting with "output-"
+     */
+    if (0 == strncmp(path, "output-", strlen("output-"))) {
+        memset(&st, 0, sizeof(struct stat));
+        fullpath = pmix_os_path(false, root, path, NULL);
+        stat(fullpath, &st);
+        free(fullpath);
+        if (0 == st.st_size) {
+            return true;
+        }
+        return false;
+    }
+
+    return true;
+}
+
 static pmix_status_t pmix_ptl_close(void)
 {
     int rc;
+    char *tmp;
 
     if (!pmix_ptl_base.initialized) {
         return PMIX_SUCCESS;
@@ -265,6 +288,17 @@ static pmix_status_t pmix_ptl_close(void)
         }
         free(pmix_ptl_base.scheduler_filename);
     }
+    if (NULL != pmix_ptl_base.sysctrlr_filename) {
+        if (pmix_ptl_base.created_sysctrlr_filename) {
+            rc = remove(pmix_ptl_base.sysctrlr_filename);
+            if (0 != rc) {
+                pmix_output_verbose(2, pmix_ptl_base_framework.framework_output,
+                                    "Remove of %s failed: %s",
+                                    pmix_ptl_base.sysctrlr_filename, strerror(errno));
+            }
+        }
+        free(pmix_ptl_base.sysctrlr_filename);
+    }
     if (NULL != pmix_ptl_base.system_filename) {
         if (pmix_ptl_base.created_system_filename) {
             rc = remove(pmix_ptl_base.system_filename);
@@ -282,7 +316,7 @@ static pmix_status_t pmix_ptl_close(void)
             if (0 != rc) {
                 pmix_output_verbose(2, pmix_ptl_base_framework.framework_output,
                                     "Remove of %s failed: %s",
-                                    pmix_ptl_base.system_filename, strerror(errno));
+                                    pmix_ptl_base.session_filename, strerror(errno));
             }
         }
         free(pmix_ptl_base.session_filename);
@@ -293,7 +327,7 @@ static pmix_status_t pmix_ptl_close(void)
             if (0 != rc) {
                 pmix_output_verbose(2, pmix_ptl_base_framework.framework_output,
                                     "Remove of %s failed: %s",
-                                    pmix_ptl_base.system_filename, strerror(errno));
+                                    pmix_ptl_base.nspace_filename, strerror(errno));
             }
         }
         free(pmix_ptl_base.nspace_filename);
@@ -304,18 +338,23 @@ static pmix_status_t pmix_ptl_close(void)
             if (0 != rc) {
                 pmix_output_verbose(2, pmix_ptl_base_framework.framework_output,
                                     "Remove of %s failed: %s",
-                                    pmix_ptl_base.system_filename, strerror(errno));
+                                    pmix_ptl_base.pid_filename, strerror(errno));
             }
         }
         free(pmix_ptl_base.pid_filename);
     }
     if (NULL != pmix_ptl_base.rendezvous_filename) {
+        if (pmix_ptl_base.created_rendezvous_dir) {
+            tmp = pmix_dirname(pmix_ptl_base.rendezvous_filename);
+            pmix_os_dirpath_destroy(tmp, true, _check_file);
+            free(tmp);
+        }
         if (pmix_ptl_base.created_rendezvous_file) {
             rc = remove(pmix_ptl_base.rendezvous_filename);
             if (0 != rc) {
                 pmix_output_verbose(2, pmix_ptl_base_framework.framework_output,
                                     "Remove of %s failed: %s",
-                                    pmix_ptl_base.system_filename, strerror(errno));
+                                    pmix_ptl_base.rendezvous_filename, strerror(errno));
             }
         }
         free(pmix_ptl_base.rendezvous_filename);
@@ -330,7 +369,7 @@ static pmix_status_t pmix_ptl_close(void)
             if (0 != rc) {
                 pmix_output_verbose(2, pmix_ptl_base_framework.framework_output,
                                     "Remove of %s failed: %s",
-                                    pmix_ptl_base.system_filename, strerror(errno));
+                                    pmix_ptl_base.urifile, strerror(errno));
             }
         }
         free(pmix_ptl_base.urifile);
@@ -339,13 +378,13 @@ static pmix_status_t pmix_ptl_close(void)
     if (NULL != pmix_ptl_base.session_tmpdir) {
         /* if I created the session tmpdir, then remove it if empty */
         if (pmix_ptl_base.created_session_tmpdir) {
-            pmix_os_dirpath_destroy(pmix_ptl_base.session_tmpdir, true, NULL);
+            pmix_os_dirpath_destroy(pmix_ptl_base.session_tmpdir, true, _check_file);
         }
         free(pmix_ptl_base.session_tmpdir);
     }
     if (NULL != pmix_ptl_base.system_tmpdir) {
         if (pmix_ptl_base.created_system_tmpdir) {
-            pmix_os_dirpath_destroy(pmix_ptl_base.system_tmpdir, true, NULL);
+            pmix_os_dirpath_destroy(pmix_ptl_base.system_tmpdir, true, _check_file);
         }
         free(pmix_ptl_base.system_tmpdir);
     }
