@@ -8,7 +8,7 @@
  * Copyright (c) 2016-2020 IBM Corporation.  All rights reserved.
  * Copyright (c) 2016-2018 Research Organization for Information Science
  *                         and Technology (RIST).  All rights reserved.
- * Copyright (c) 2021-2023 Nanook Consulting.  All rights reserved.
+ * Copyright (c) 2021-2025 Nanook Consulting  All rights reserved.
  * $COPYRIGHT$
  */
 
@@ -122,6 +122,7 @@ typedef struct {
     pmix_event_t ev;
     bool event_active;
     pmix_dmdx_local_t *lcd;
+    char *key;
     pmix_modex_cbfunc_t cbfunc; // cbfunc to be executed when data is available
     void *cbdata;
 } pmix_dmdx_request_t;
@@ -269,7 +270,14 @@ PMIX_EXPORT pmix_status_t pmix_server_unpublish(pmix_peer_t *peer, pmix_buffer_t
 
 PMIX_EXPORT pmix_status_t pmix_server_spawn(pmix_peer_t *peer, pmix_buffer_t *buf,
                                             pmix_spawn_cbfunc_t cbfunc, void *cbdata);
-PMIX_EXPORT void pmix_server_spawn_parser(pmix_peer_t *peer, pmix_setup_caddy_t *cd);
+PMIX_EXPORT void pmix_server_spawn_parser(pmix_peer_t *peer,
+                                          pmix_iof_channel_t *channels,
+                                          pmix_iof_flags_t *flags,
+                                          pmix_info_t *info,
+                                          size_t ninfo);
+PMIX_EXPORT pmix_status_t pmix_server_process_iof(pmix_setup_caddy_t *cd,
+                                                  char nspace[]);
+
 PMIX_EXPORT void pmix_server_spcbfunc(pmix_status_t status, char nspace[], void *cbdata);
 
 PMIX_EXPORT pmix_status_t pmix_server_connect(pmix_server_caddy_t *cd, pmix_buffer_t *buf,
@@ -358,9 +366,17 @@ PMIX_EXPORT pmix_status_t pmix_server_refresh_cache(pmix_server_caddy_t *cd,
                                                     pmix_buffer_t *buf,
                                                     pmix_op_cbfunc_t cbfunc);
 
-PMIX_EXPORT void pmix_server_query_cbfunc(pmix_status_t status,
-                                          pmix_info_t *info, size_t ninfo, void *cbdata,
-                                          pmix_release_cbfunc_t release_fn, void *release_cbdata);
+PMIX_EXPORT pmix_status_t pmix_server_resolve_peers(pmix_server_caddy_t *cd,
+                                                    pmix_buffer_t *buf,
+                                                    pmix_info_cbfunc_t cbfunc);
+
+PMIX_EXPORT void pmix_server_locally_resolve_peers(int sd, short args, void *cbdata);
+
+PMIX_EXPORT pmix_status_t pmix_server_resolve_node(pmix_server_caddy_t *cd,
+                                                   pmix_buffer_t *buf,
+                                                   pmix_info_cbfunc_t cbfunc);
+
+PMIX_EXPORT void pmix_server_locally_resolve_node(int sd, short args, void *cbdata);
 
 PMIX_EXPORT extern pmix_server_module_t pmix_host_server;
 PMIX_EXPORT extern pmix_server_globals_t pmix_server_globals;
